package ru.ifmo.fileShare;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

public class Consts {
    public static final int SERVER_PORT;
    public static final int SERVER_TIMEOUT;
    public static final int CLIENT_TIMEOUT;
    public static final boolean CLOSE_TABS_AFTER_DISCONNECT;

    protected static final String PROPERTIES_FILE = "NetFileShare.properties";
    protected static final int DEFAULT_PORT = 6001;
    protected static final int DEFAULT_SERVER_TIMEOUT = 30000;
    protected static final int DEFAULT_CLIENT_TIMEOUT = 30000;
    protected static final boolean DEFAULT_CLOSE_TABS_AFTER_DISCONNECT = false;

    static {
        Properties props = new Properties();
        int port;
        int clientTimeout;
        int serverTimeout;
        boolean closeTabsAfterDisconnect;
        try {
            props.load(new FileInputStream(PROPERTIES_FILE));
            port = Integer.parseInt(props.getProperty("server-port"));
            clientTimeout = Integer.parseInt(props.getProperty("client-timeout"));
            serverTimeout = Integer.parseInt(props.getProperty("server-timeout"));
            closeTabsAfterDisconnect = Boolean.parseBoolean(props.getProperty("close-tabs-after-disconnect"));
        } catch (IOException e) {
            port = DEFAULT_PORT;
            clientTimeout = DEFAULT_CLIENT_TIMEOUT;
            serverTimeout = DEFAULT_SERVER_TIMEOUT;
            closeTabsAfterDisconnect = DEFAULT_CLOSE_TABS_AFTER_DISCONNECT;
        }
        SERVER_PORT = port;
        SERVER_TIMEOUT = serverTimeout;
        CLIENT_TIMEOUT = clientTimeout;
        CLOSE_TABS_AFTER_DISCONNECT = closeTabsAfterDisconnect;
    }

    public static final String HOSTNAME_PARAM = "hostname";
    public static final String PORT_PARAM = "port";
    public static final String FILEPART_PARAM = "filePart";
    public static final String LISTPART_PARAM = "listPart";
    public static final String FOLDER_ID_PARAM = "folderId";
    public static final String FILE_ID_PARAM = "fileId";
    public static final String OUTPUT_FILENAME_PARAM = "outputFilename";
    public static final String CLIENT_HOSTNAME_PARAM = "clientHostname";
    public static final String CLIENT_IP_PARAM = "clientIP";
    public static final String CLIENT_CONNECT_DATE_PARAM = "clientConnectDate";
    public static final String CLIENT_STATUS_PARAM = "clientStatus";

    public static final int ROOT_FOLDER_ID = -1;
    public static final int FILE_PART_SIZE = 32768;
    public static final int LIST_PART_SIZE = 32768;
}
