package ru.ifmo.fileShare.client;

import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.gui.SharingApp;
import ru.ifmo.fileShare.gui.controls.WaitDialog;
import ru.ifmo.fileShare.transport.FileListItem;
import ru.ifmo.fileShare.util.FolderInfo;
import ru.ifmo.fileShare.util.Progress;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

public class BrowserResourceController {
    private List<FolderInfo> folderInfo;
    private long loadingForFileID;
    private WaitDialog waitDialog;
    private boolean forDownload;
    private FolderInfo fileDownloading;
    private boolean canceling;

    public BrowserResourceController() {
        folderInfo = new ArrayList<FolderInfo>();
    }

    public List<FolderInfo> getFolderInfo() {
        return folderInfo;
    }

    public void startLoadSubFolders(FolderInfo fi) {
        startLoadSubFolders(fi, false);
    }

    public void startLoadSubFolders(FolderInfo fi, boolean forDownload) {
        this.forDownload = forDownload;
        startLoadSubFolders(fi.getFileID());
    }

    public void startLoadRootFolders() {
        loadingForFileID = -1;
        ClientUserEventProvider.getInstance().getRootList();
    }

    public void startLoadSubFolders(long fileID) {
        loadingForFileID = fileID;
        ClientUserEventProvider.getInstance().getList(fileID);
    }

    public FolderInfo getFolderInfoByID(long id) {
        FolderInfo f = null;
        for (FolderInfo fi : folderInfo) {
            FolderInfo found = fi.getByID(id);
            if (found != null) {
                f = found;
                break;
            }
        }
        return f;
    }

    public boolean fileListUpdate(FileListItem[] fileListItems) {
        List<FolderInfo> children = FolderInfo.wrap(fileListItems);
        if (loadingForFileID == Consts.ROOT_FOLDER_ID) {
            folderInfo = children;
        } else {
            FolderInfo found = getFolderInfoByID(loadingForFileID);
            if (found == null) {
                JOptionPane.showMessageDialog(null, "Loaded file list for wrong ID", "warning", JOptionPane.WARNING_MESSAGE);
            } else {
                found.setChildren(children);
                found.setLoaded(true);
            }
        }
        if (forDownload) {
            waitDialog.close();
        }
        return !forDownload;
    }

    public void clearFolderInfo() {
        folderInfo = new ArrayList<FolderInfo>();
    }

    public List<FolderInfo> getSelected() {
        List<FolderInfo> selected = new ArrayList<FolderInfo>();
        for (FolderInfo fi : folderInfo) {
            fi.getSelected(selected);
        }
        return selected;
    }

    public void startDownload(List<FolderInfo> selected, String copyTo) {
        loadAllSubFolderInfo(selected);
        startDownloadFiles(selected, copyTo);
    }

    private void startDownloadFiles(List<FolderInfo> selected, String copyTo) {
        canceling = false;
        List<FolderInfo> files = getFullFileList(selected);
        downloadList(files, copyTo);
        for (FolderInfo fi : selected) {
            fi.setSelected(false);
        }
    }

    private void downloadList(List<FolderInfo> files, String copyTo) {
        int counter = 0;
        int count = files.size();
        for (FolderInfo fi : files) {
            counter++;
            if (canceling) {
                break;
            }
            Progress progress = new Progress() {
                public void cancel() {
                    canceling = true;
                }
            };

            waitDialog = new WaitDialog(SharingApp.getInstance(), progress);
            fileDownloading = fi;
            waitDialog.setTitle("(" + counter + "/" + count + ") " + "Downloading file:" + fileDownloading.getRelativePath());
            waitDialog.setIndeterminate(false);
            startFileDownload(fi, copyTo);
            waitDialog.start();
        }
    }

    private void startFileDownload(FolderInfo fi, String copyTo) {
        FolderInfo parentSelected = fi.getParentSelected();
        String pathToParent = parentSelected.getPath();
        String relative = fi.getRelativePath(pathToParent);
        if (parentSelected.isDirectory()) {
            relative = parentSelected.getName() + "\\" + relative;
        }
        ClientUserEventProvider.getInstance().getFile(fi.getFileID(), copyTo + "\\" + relative);
    }

    private void loadAllSubFolderInfo(List<FolderInfo> selected) {
        for (FolderInfo fi : selected) {
            FolderInfo child = fi.getNotLoadedChild();
            while (child != null) {
                waitDialog = new WaitDialog(SharingApp.getInstance(), null);
                waitDialog.setTitle("Processing folder: " + child.getRelativePath());
                startLoadSubFolders(child, true);
                waitDialog.start();
                child = fi.getNotLoadedChild();
            }
        }
    }

    private List<FolderInfo> getFullFileList(List<FolderInfo> selected) {
        List<FolderInfo> files = new ArrayList<FolderInfo>();
        for (FolderInfo fi : selected) {
            fi.getFiles(files);
        }
        return files;
    }

    public void onFileReceived() {
        fileDownloading.setSelected(false);
        waitDialog.close();
    }

    public void onFilePartReceived(long downloadedSize) {
        waitDialog.setProgress((float) downloadedSize / fileDownloading.getLength());
        if (canceling) {
            ClientUserEventProvider.getInstance().cancelFileDownload();
            waitDialog.close();
        }
    }
}
