package ru.ifmo.fileShare.client;

import com.evelopers.unimod.runtime.ControlledObject;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.gui.SharingApp;
import ru.ifmo.fileShare.transport.FileListItem;
import ru.ifmo.fileShare.transport.TransportUtil;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

public class ClientController implements ControlledObject {

    private ClientTimer timer;
    private ClientSocket socket;
    private FileOutputStream fileStream;
    private String outputFilename;
    private long downloadedSize;
    private byte[] list;

    /**
     * @param context state machine context
     * @unimod.action.descr say hello to server
     */
    public void z1(StateMachineContext context) {
        socket.sayHello();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle connection error
     */
    public void z2(StateMachineContext context) {
        SharingApp.getInstance().getClientEventListener().onConnectionError(socket.getLastError());
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle server said hello
     */
    public void z3(StateMachineContext context) {
        SharingApp.getInstance().getClientEventListener().onConnectedToServer();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr request first list part
     */
    public void z4(StateMachineContext context) {
        socket.sayFirstListPart((Long) context.getEventContext().getParameter(Consts.FOLDER_ID_PARAM));
    }

    /**
     * @param context state machine context
     * @unimod.action.descr request next list part
     */
    public void z5(StateMachineContext context) {
        socket.sayNextListPart();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle received list part
     */
    public void z6(StateMachineContext context) {
        addToList((byte[]) context.getEventContext().getParameter(Consts.LISTPART_PARAM));
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle received last list part
     */
    public void z7(StateMachineContext context) {
        addToList((byte[]) context.getEventContext().getParameter(Consts.LISTPART_PARAM));
        FileListItem[] fileListItems = null;
        try {
            fileListItems = (FileListItem[]) TransportUtil.fromByteArray(list);
        } catch (IOException e) {
            e.printStackTrace();
        }
        SharingApp.getInstance().getClientEventListener().onListRecieved(fileListItems);
        list = null;
    }

    /**
     * @param context state machine context
     * @unimod.action.descr request first file part
     */
    public void z8(StateMachineContext context) {
        outputFilename = (String) context.getEventContext().getParameter(Consts.OUTPUT_FILENAME_PARAM);
        socket.sayFirstFilePart((Long) context.getEventContext().getParameter(Consts.FILE_ID_PARAM));
    }

    /**
     * @param context state machine context
     * @unimod.action.descr request next file part
     */
    public void z9(StateMachineContext context) {
        socket.sayNextFilePart();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr cancel file downloading
     */
    public void z10(StateMachineContext context) {
        socket.sayCancelDownload();
        closeStream(true);
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle received file part
     */
    public void z11(StateMachineContext context) {
        try {
            handleFilePart((byte[]) context.getEventContext().getParameter(Consts.FILEPART_PARAM));
            SharingApp.getInstance().getClientEventListener().onFilePartRecieved(downloadedSize);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle received last file part
     */
    public void z12(StateMachineContext context) {
        try {
            handleFilePart((byte[]) context.getEventContext().getParameter(Consts.FILEPART_PARAM));
        } catch (IOException e) {
            e.printStackTrace();
        }
        closeStream(false);
        SharingApp.getInstance().getClientEventListener().onFileRecieved();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr say bye to server and close connection
     */
    public void z13(StateMachineContext context) {
        socket.sayBye();
        socket.closeConnection();
        closeStream(true);
        SharingApp.getInstance().getClientEventListener().onSessionClosed();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr start timeout timer
     */
    public void z14(StateMachineContext context) {
        timer.startTimer(context);
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle timeout
     */
    public void z15(StateMachineContext context) {
        SharingApp.getInstance().getClientEventListener().onServerDropped();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr stop timeout timer
     */
    public void z16(StateMachineContext context) {
        timer.clearTimer();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle server closed session
     */
    public void z17(StateMachineContext context) {
        closeStream(true);
        socket.closeConnection();
        SharingApp.getInstance().getClientEventListener().onServerDisconnect();
    }

    /**
     * @param context state machine context
     * @unimod.action.descr handle quit
     */
    public void z18(StateMachineContext context) {
        closeStream(true);
        SharingApp.getInstance().getClientEventListener().onClientQuited();
    }

    /**
     * @param context state machine context
     * @return value indicating whether connection was successful or not
     * @unimod.action.descr was connection successful
     */
    public boolean x1(StateMachineContext context) {
        return socket.open((String) context.getEventContext().getParameter(Consts.HOSTNAME_PARAM),
                Integer.parseInt((String) context.getEventContext().getParameter(Consts.PORT_PARAM)));
    }

    public void initTimer(ClientTimer timer) {
        this.timer = timer;
    }

    public void initSocket(ClientSocket socket) {
        this.socket = socket;
    }

    private void handleFilePart(byte[] part) throws IOException {
        if (fileStream == null) {
            initFileStream();
        }
        fileStream.write(part);
        downloadedSize += part.length;
    }

    private void initFileStream() throws IOException {
        downloadedSize = 0;
        File file = new File(outputFilename);
        if (!file.getParentFile().exists()) {
            createRecursively(file);
        }
        fileStream = new FileOutputStream(file);
    }

    private void createRecursively(File file) throws IOException {
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
//            createRecursively(file.getParentFile());
        }
        file.createNewFile();
    }

    private void addToList(byte[] data) {
        if (list == null) {
            list = new byte[data.length];
            System.arraycopy(data, 0, list, 0, data.length);
        } else {
            byte[] newList = new byte[list.length + data.length];
            System.arraycopy(list, 0, newList, 0, list.length);
            System.arraycopy(data, 0, newList, list.length, data.length);
            list = newList;
        }
    }

    private void closeStream(boolean deleteFile) {
        if (fileStream != null) {
            try {
                fileStream.close();
                if (deleteFile) {
                    File file = new File(outputFilename);
                    file.delete();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            fileStream = null;
        }
    }
}
