package ru.ifmo.fileShare.client;

import com.evelopers.common.exception.CommonException;
import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.EventProvider;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;
import ru.ifmo.fileShare.Consts;

public class ClientUserEventProvider implements EventProvider {

    /**
     * @unimod.event.descr open connection
     */
    public static final String E111 = "e111";

    /**
     * @unimod.event.descr get list
     */
    public static final String E112 = "e112";

    /**
     * @unimod.event.descr get file
     */
    public static final String E113 = "e113";

    /**
     * @unimod.event.descr cancel file downloading
     */
    public static final String E114 = "e114";

    /**
     * @unimod.event.descr close connection
     */
    public static final String E115 = "e115";

    /**
     * @unimod.event.descr quit
     */
    public static final String E116 = "e116";

    private ModelEngine engine;

    protected static ClientUserEventProvider instance;

    public ClientUserEventProvider() {
        instance = this;
    }

    public static ClientUserEventProvider getInstance() {
        return instance;
    }

    public void init(ModelEngine engine) throws CommonException {
        this.engine = engine;
    }

    public void connect(String hostname, String port) {
        StateMachineContext context = StateMachineContextImpl.create();
        context.getEventContext().setParameter(Consts.HOSTNAME_PARAM, hostname);
        context.getEventContext().setParameter(Consts.PORT_PARAM, port);
        engine.getEventManager().handle(new Event(E111), context);
    }

    public void disconnect() {
        engine.getEventManager().handle(new Event(E115), StateMachineContextImpl.create());
    }

    public void dispose() {
    }

    public void quit() {
        engine.getEventManager().handle(new Event(E116), StateMachineContextImpl.create());
    }

    public void cancelFileDownload() {
        engine.getEventManager().handle(new Event(E114), StateMachineContextImpl.create());
    }

    public void getRootList() {
        getList(Consts.ROOT_FOLDER_ID);
    }

    public void getList(long folderId) {
        StateMachineContext context = StateMachineContextImpl.create();
        context.getEventContext().setParameter(Consts.FOLDER_ID_PARAM, folderId);
        engine.getEventManager().handle(new Event(E112), context);
    }

    public void getFile(long fileId, String outputFileName) {
        StateMachineContext context = StateMachineContextImpl.create();
        context.getEventContext().setParameter(Consts.FILE_ID_PARAM, fileId);
        context.getEventContext().setParameter(Consts.OUTPUT_FILENAME_PARAM, outputFileName);
        engine.getEventManager().handle(new Event(E113), context);
    }
}
