package ru.ifmo.fileShare.gui;

import ru.ifmo.fileShare.gui.controls.FolderInfoTreeNode;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.io.File;
import java.util.ResourceBundle;

public abstract class BaseResourcesPane extends JPanel {
    protected ResourceBundle props;
    private File lastDir;
    protected FolderTreeModel folderTreeModel;
    protected FileTableModel fileTableModel;
    protected JTree folderTree;
    protected JTable filesTable;
    protected JLabel status;

    protected void create() {
        JComponent pane = createMainPane();
        JComponent toolPane = createToolPane();
        add(pane, BorderLayout.CENTER);
        add(toolPane, BorderLayout.NORTH);
        setupControls();
        updateControlState();
    }

    protected JComponent createToolPane() {
        return new JPanel();
    }

    protected JComponent createMainPane() {
        JSplitPane pane = new JSplitPane();
        pane.setDividerLocation(300);
        pane.setLeftComponent(createFolderTree());
        JComponent list = createFileList();
        JPanel rightPane = new JPanel(new BorderLayout());
        rightPane.add(list, BorderLayout.CENTER);
        status = new JLabel();
        status.setBorder(new EmptyBorder(2, 5, 2, 5));
        rightPane.add(status, BorderLayout.SOUTH);
        pane.setRightComponent(rightPane);
        setLayout(new BorderLayout());
        return pane;
    }

    protected void updateControlState() {
        updateFileListStatus();
    }

    protected void setupControls() {
    }

    private JComponent createFileList() {
        filesTable = new JTable();
        filesTable.setRowHeight(20);
        filesTable.setFont(new Font("Arial", Font.PLAIN, 12));
        filesTable.setModel(createFileTableModel());
        return new JScrollPane(filesTable);
    }

    private FileTableModel createFileTableModel() {
        fileTableModel = new FileTableModel(allowFileSelection());
        return fileTableModel;
    }

    protected boolean allowFileSelection() {
        return false;
    }

    private JComponent createFolderTree() {
        folderTree = new JTree();
        folderTree.setRowHeight(20);
        folderTree.setFont(new Font("Arial", Font.PLAIN, 12));
        folderTree.setModel(createFolderTreeModel());
        folderTree.setShowsRootHandles(true);
        folderTree.setRootVisible(false);
        folderTree.getSelectionModel().addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                TreePath selectionPath = folderTree.getSelectionPath();
                Object selected = null;
                if (selectionPath != null) {
                    FolderInfoTreeNode node = (FolderInfoTreeNode) selectionPath.getLastPathComponent();
                    if (node != null) {
                        selected = node.getUserObject();
                    }
                }
                onTreeSelectionChanged(selected);
            }
        });

//        folderTree.setCellRenderer(new FolderTreeCellRenderer());
//        folderTree.setCellEditor(new FolderTreeCellEditor(folderTree));
//        folderTree.setEditable(true);

        return new JScrollPane(folderTree);
    }

    protected void onTreeSelectionChanged(Object userObject) {

    }

    private TreeModel createFolderTreeModel() {
        folderTreeModel = new FolderTreeModel();
        return folderTreeModel;
    }

    protected File getLastDir() {
        if (lastDir == null) {
            lastDir = File.listRoots()[0];
        }
        return lastDir;
    }

    protected void setLastDir(File lastDir) {
        this.lastDir = lastDir;
    }

    protected void updateFileListStatus() {
        status.setText(fileTableModel.getRowCount() + " files.");
    }
}
