package ru.ifmo.fileShare.gui;

import ru.ifmo.fileShare.ServerEventListener;
import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.util.ClientInfo;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

public class ClientInfoPane extends JPanel implements ServerEventListener {
    protected ClientInfo ci;
    private ClientInfoTableModel clientInfoModel;
    private ResourceBundle props;
    private JButton disconnectButton;
    private JButton closeTabButton;
    private boolean serverQuited;

    public ClientInfoPane() {
        props = ResourceBundle.getBundle("ru.ifmo.fileShare.gui.form");
        create();
    }

    protected void create() {
        setLayout(new BorderLayout());
        JTable clientInfoTable = createClientInfoTable();
        add(clientInfoTable, BorderLayout.CENTER);
        JPanel bottomPane = new JPanel(new FlowLayout());
        disconnectButton = new JButton(props.getString("disconnect"));
        disconnectButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                doDisconnect();
            }
        });
        bottomPane.add(disconnectButton);
        if (!Consts.CLOSE_TABS_AFTER_DISCONNECT) {
            closeTabButton = new JButton(props.getString("close_tab"));
            closeTabButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    closeTab();
                }
            });
            closeTabButton.setEnabled(false);
            bottomPane.add(closeTabButton);
        }
        add(bottomPane, BorderLayout.SOUTH);
        updateControlState();
        serverQuited = false;
    }

    private void closeTab() {
        SharingApp.getInstance().removeClientInfoPane(this);
    }

    private void doDisconnect() {
        SharingApp.getInstance().getServerUserEventProvider(this).disconnect();
    }
    
    private void updateControlState() {
        disconnectButton.setEnabled(isClientConnected());
    }

    private JTable createClientInfoTable() {
        JTable table = new JTable();
        clientInfoModel = new ClientInfoTableModel();
        table.setModel(clientInfoModel);
        table.setFont(new Font("Arial", Font.BOLD, 16));
        table.setRowHeight(30);
        return table;
    }

    public void onClientConnected(ClientInfo ci) {
        this.ci = ci;
        updateClientInfo();
        updateControlState();
    }

    public void onSessionClosed() {
        serverQuited = true;
        disconnectButton.setEnabled(false);
        if (Consts.CLOSE_TABS_AFTER_DISCONNECT) {
            closeTab();
        } else {
            closeTabButton.setEnabled(true);
        }
    }

    public void onActionUpdate(String action) {
        ci.setActions(action);
        updateClientInfo();
    }

    public boolean hasServerQuited() {
        return serverQuited;
    }

    private void updateClientInfo() {
        clientInfoModel.setClientInfo(ci);
    }

    private boolean isClientConnected() {
        return ci != null;
    }
}
