package ru.ifmo.fileShare.gui;

import ru.ifmo.fileShare.util.FolderInfo;

import javax.swing.table.AbstractTableModel;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class FileTableModel extends AbstractTableModel {

    protected List<FolderInfo> files;
    private final static String[] columns = {"", "File Name", "Modified", "Size"};
    private boolean allowFileSelection;

    public FileTableModel(boolean allowFileSelection) {
        this.allowFileSelection = allowFileSelection;
        files = Collections.emptyList();
    }

    public int getRowCount() {
        return files.size();
    }

    public int getColumnCount() {
        return 4 - (allowFileSelection ? 0 : 1);
    }

    public String getColumnName(int column) {
        return columns[column + (allowFileSelection ? 0 : 1)];
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        FolderInfo fi = files.get(rowIndex);
        switch (columnIndex + (allowFileSelection ? 0 : 1)) {
            case 0:
                return fi;
            case 1:
                return fi.getName();
            case 2:
                return DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT).format(fi.getLastModified());
            case 3:
                return fi.getLength();
        }
        return null;
    }


    public boolean isCellEditable(int rowIndex, int columnIndex) {
        FolderInfo fi = files.get(rowIndex);
        return allowFileSelection && columnIndex == 0 && !fi.isDisabled();
    }


    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        FolderInfo fi = files.get(rowIndex);
        fi.setSelected((Boolean) aValue);
    }

    public void updateData() {
        fireTableDataChanged();
    }

    public void updateData(FolderInfo selectedFolder) {
        files = new ArrayList<FolderInfo>();
        if (selectedFolder != null) {
            if (selectedFolder.getChildren() != null) {
                for (FolderInfo fi : selectedFolder.getChildren()) {
                    if (!fi.isDirectory()) {
                        files.add(fi);
                    }
                }
            }
        }
        fireTableDataChanged();
    }

    public void selectAll(boolean b) {
        for (FolderInfo fi : files) {
            if (!fi.isDisabled()) {
                fi.setSelected(b);
            }
        }
        updateData();
    }
}
