package ru.ifmo.fileShare.gui;

import ru.ifmo.fileShare.util.FolderInfo;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;

public class SharedResourcesPane extends BaseResourcesPane {
    private JButton addFolderButton;
    private JButton removeFolderButton;
    private FolderInfo selectedFolder;

    public SharedResourcesPane() {
        props = ResourceBundle.getBundle("ru.ifmo.fileShare.gui.form");
    }

    protected JComponent createToolPane() {
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.LEADING));
        addFolderButton = new JButton(props.getString("add_folder"));
        buttonPane.add(addFolderButton);
        removeFolderButton = new JButton(props.getString("remove_folder"));
        buttonPane.add(removeFolderButton);
        return buttonPane;
    }

    protected void updateControlState() {
        super.updateControlState();
        if (removeFolderButton != null) {
            removeFolderButton.setEnabled(isFolderSelected());
        }
    }

    private boolean isFolderSelected() {
        return selectedFolder != null;
    }

    protected void setupControls() {
        addFolderButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onFolderAdd();
            }
        });
        removeFolderButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onFolderRemove();
            }
        });
        super.setupControls();
    }

    private void onFolderRemove() {
        SharingApp.getInstance().getSharedResourceController().removeSharedFolder(selectedFolder);
        updateData();
    }

    private void onFolderAdd() {
        JFileChooser jf = new JFileChooser();
        jf.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        jf.setCurrentDirectory(getLastDir());
        if (JFileChooser.APPROVE_OPTION == jf.showOpenDialog(this)) {
            File f = jf.getSelectedFile();
            if (f != null) {
                setLastDir(jf.getSelectedFile().getParentFile());
                SharingApp.getInstance().getSharedResourceController().addSharedFolder(
                        f.getAbsolutePath(), SharingApp.getInstance(), new Runnable() {
                    public void run() {
                        updateData();
                    }
                }
                );
            }
        }
    }

    public void updateData() {
        folderTreeModel.updateData(SharingApp.getInstance().getSharedResourceController().getSharedFolderInfo());
    }

    protected void onTreeSelectionChanged(Object userObject) {
        selectedFolder = (FolderInfo) userObject;
        fileTableModel.updateData(selectedFolder);
        updateFileListStatus();
        updateControlState();
    }
}
