package ru.ifmo.fileShare.gui.controls;

import javax.swing.*;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.TreeCellEditor;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.EventObject;

public class FolderTreeCellEditor implements TreeCellEditor {
    private java.util.List<CellEditorListener> listeners;
    private FolderTreeCellRenderer renderer;
    private Point point;
    private ActionListener listener;
    private JTree tree;

    public FolderTreeCellEditor(JTree tree) {
        this.tree = tree;
        listeners = new ArrayList<CellEditorListener>();
        renderer = new FolderTreeCellRenderer();
        listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stopCellEditing();
            }
        };
    }

    public Component getTreeCellEditorComponent(final JTree tree, Object value, boolean isSelected, boolean expanded, boolean leaf, int row) {
        FolderInfoTreeNode node = (FolderInfoTreeNode) value;
        Rectangle rect = tree.getRowBounds(row);
        if (rect.contains(point)) {
            if (point.x - rect.x < 20) {
                node.toggleSelection();
            }
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                stopCellEditing();
                tree.stopEditing();
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        tree.repaint();
                    }
                });
            }
        });
        return renderer.getTreeCellRendererComponent(tree, value, isSelected, expanded, leaf, row, true);
    }

    public Object getCellEditorValue() {
        return null;
    }

    public boolean isCellEditable(EventObject anEvent) {
        if (anEvent instanceof MouseEvent) {
            point = ((MouseEvent) anEvent).getPoint();
            int row = tree.getRowForLocation(point.x, point.y);
            Rectangle rect = tree.getRowBounds(row);
            if (rect.contains(point)) {
                if (point.x - rect.x < 20) {
                    return true;
                }
            }
        }
        return false;
    }

    public boolean shouldSelectCell(EventObject anEvent) {
        return false;
    }

    public boolean stopCellEditing() {
        return true;
    }

    public void cancelCellEditing() {
        renderer.cbx.removeActionListener(listener);
        ChangeEvent changeEvent = new ChangeEvent(this);
        for (CellEditorListener l : listeners) {
            l.editingCanceled(changeEvent);
        }
    }

    public void addCellEditorListener(CellEditorListener l) {
        listeners.add(l);
    }

    public void removeCellEditorListener(CellEditorListener l) {
        listeners.remove(l);
    }
}
