package ru.ifmo.fileShare.gui.controls;

import ru.ifmo.fileShare.util.Progress;
import ru.ifmo.fileShare.util.WindowUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;

public class WaitDialog extends JDialog {
    protected ResourceBundle props;
    private Progress progress;
    private JProgressBar progressBar;

    public WaitDialog(JFrame parent, Progress progress) {
        super(parent);
        props = ResourceBundle.getBundle("ru.ifmo.fileShare.gui.form");
        this.progress = progress;
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        create();
    }

    private void create() {
        getContentPane().setLayout(new FlowLayout(FlowLayout.LEADING));
        progressBar = new JProgressBar();
        setTitle(props.getString("wait"));
        setModal(true);
        setResizable(false);
        getContentPane().add(progressBar);
        progressBar.setIndeterminate(true);
        progressBar.setPreferredSize(new Dimension(275, 20));
        final JButton cancelButton = new JButton(props.getString("cancel"));
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                progress.cancel();
                cancelButton.setEnabled(false);
            }
        });
        getContentPane().add(cancelButton);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                if (progress != null) {
                    progress.cancel();
                }
            }
        });
        if (progress == null) {
            cancelButton.setEnabled(false);
        }
        setSize(new Dimension(370, 65));
    }

    public void start() {
        WindowUtils.locateAtScreenCenter(this);
        setVisible(true);
    }

    public void close() {
        dispose();
    }

    public void setIndeterminate(boolean b) {
        progressBar.setIndeterminate(b);
    }

    public void setProgress(float v) {
        progressBar.setMaximum(100);
        progressBar.setValue((int) (100 * v));
    }
}
