package ru.ifmo.fileShare.server;

import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.math.NumberUtils;
import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.ServerEventListener;
import ru.ifmo.fileShare.ServerModelEventProcessor;
import ru.ifmo.fileShare.gui.ClientInfoPane;
import ru.ifmo.fileShare.gui.SharingApp;

import java.io.IOException;
import java.net.Socket;
import java.net.SocketException;

public class ServerSocketAcceptor implements Runnable {

    private static Socket connectingSocket;
    private static java.net.ServerSocket serverSocket;
    private static boolean socketCreated;
    private static boolean eventListenerCreated;
    private static boolean eventProviderCreated;
    private static ClientInfoPane createdClientInfoPane;
    private static ServerUserEventProvider createdServerUserEventProvider;

    public static void startSocketAcceptor() {
        Thread acceptorThread = new Thread(new ServerSocketAcceptor());
        acceptorThread.start();
    }

    public static Socket getConnectingSocket() {
        socketCreated = true;
        return connectingSocket;
    }

    public static ServerEventListener getConnectingEventListener() {
        createdClientInfoPane = SharingApp.getInstance().createConnectedClientsPane();
        eventListenerCreated = true;
        if (eventProviderCreated) {
            updateGui();
        }
        return createdClientInfoPane;
    }

    public static void setConnectingServerUserEventProvider(ServerUserEventProvider suep) {
        createdServerUserEventProvider = suep;
        eventProviderCreated = true;
        if (eventListenerCreated) {
            updateGui();
        }
    }

    private static void updateGui() {
        SharingApp.getInstance().addClientInfoPane(createdClientInfoPane, createdServerUserEventProvider);
    }

    public void run() {
        try {
            serverSocket = new java.net.ServerSocket(Consts.SERVER_PORT);
            while (serverSocket != null) {
                try {
                    acceptSocket(serverSocket.accept());
                } catch (SocketException se) {
                    // ignoring socket closed exception
                    if (se.getMessage().indexOf("Socket is closed") == -1 &&
                            se.getMessage().indexOf("socket closed") == -1) {
                        se.printStackTrace();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static synchronized void acceptSocket(Socket socket) {
        connectingSocket = socket;
        socketCreated = false;
        eventListenerCreated = false;
        eventProviderCreated = false;
        try {
//          Run.main(new String[] {"A2.xml"});
            int debuggerPort = NumberUtils.stringToInt(System.getProperty("debugger.port"), -1);
            boolean debuggerSuspend = BooleanUtils.toBoolean(System.getProperty("debugger.suspend"));
            ServerModelEventProcessor.run(debuggerPort, debuggerSuspend);
        } catch (Exception e) {
            e.printStackTrace();
        }
        while (!socketCreated || !eventListenerCreated || !eventProviderCreated) {
            // waiting for connection set up
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // ignore
            }
        }
    }

    public static void stopSocketAcceptor() {
        try {
            serverSocket.close();
            serverSocket = null;
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
