package ru.ifmo.fileShare.server;

import ru.ifmo.fileShare.Consts;
import ru.ifmo.fileShare.gui.controls.WaitDialog;
import ru.ifmo.fileShare.transport.FileListItem;
import ru.ifmo.fileShare.util.FilesEnumerator;
import ru.ifmo.fileShare.util.FolderEnumeratorProgress;
import ru.ifmo.fileShare.util.FolderInfo;
import ru.ifmo.fileShare.util.ServerConfiguration;

import javax.swing.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class SharedResourceController {
    protected List<String> sharedRoots;

    protected ServerConfiguration config;

    protected List<FolderInfo> sharedFolderInfo;

    private static final String SHARED_FOLDERS = "shared-folders-list";
    private WaitDialog waitDialog;
//    private static final String SELECTED_FOLDERS = "shared-folders-selected";

    public SharedResourceController() {
        config = new ServerConfiguration();
        sharedRoots = new ArrayList<String>();
        sharedFolderInfo = new ArrayList<FolderInfo>();
    }

    protected void loadSharedRoots() {
        sharedRoots = new ArrayList<String>(Arrays.asList(config.getStringList(SHARED_FOLDERS)));
    }

    protected void saveSharedRoots() {
        config.setStringList(SHARED_FOLDERS, sharedRoots.toArray(new String[0]));
    }

    public List<String> getSharedRoots() {
        return sharedRoots;
    }

    public void addSharedFolder(String path, JFrame parent, Runnable onTerminated) {
        addSharedFolder(path, parent, onTerminated, false);
    }

    protected void addSharedFolder(final String path, JFrame parent, final Runnable onTerminated, final boolean alreadyAdded) {
        FolderEnumeratorProgress progress = new FolderEnumeratorProgress() {
            public void onFinished(int state, FolderInfo folderInfo) {
                if (state == FINISHED) {
                    if (!alreadyAdded) {
                        sharedRoots.add(path);
                    }
                    sharedFolderInfo.add(folderInfo);
                }
                hideWaitDialog();
                onTerminated.run();
            }
        };
        waitDialog = new WaitDialog(parent, progress);
        FilesEnumerator.enumFoldersAsync(path, progress);
        showWaitDialog();
    }

    public void collectSharedFolderInfo(final JFrame parent, final Runnable onTerminated) {
        sharedFolderInfo.clear();
        final Iterator<String> iter = sharedRoots.iterator();
        Runnable myCounter = new Runnable() {
            public void run() {
                if (iter.hasNext()) {
                    addSharedFolder(iter.next(), parent, this, true);
                } else {
//                    applySharedFolderSelection();
                    onTerminated.run();
                }
            }
        };
        if (iter.hasNext()) {
            addSharedFolder(iter.next(), parent, myCounter, true);
        }
    }

    private void hideWaitDialog() {
        waitDialog.close();
    }

    private void showWaitDialog() {
        waitDialog.start();
    }

    public List<FolderInfo> getSharedFolderInfo() {
        return sharedFolderInfo;
    }

    public void saveSharedFolderInfo() {
        saveSharedRoots();
//        saveSharedFolderSelections();
        config.save();
    }

    public void loadSharedFolderInfo() {
        loadSharedRoots();
//        loadSharedFolderSelection();
    }

//    private void loadSharedFolderSelection() {
//        sharedFolderSelection = Arrays.asList(config.getStringList(SELECTED_FOLDERS));
//    }

//    private void saveSharedFolderSelections() {
//        sharedFolderSelection = new ArrayList<String>();
//        for (FolderInfo fi: sharedFolderInfo) {
//            updateFolderSelection(fi, sharedFolderSelection);
//        }
//        config.setStringList(SELECTED_FOLDERS, sharedFolderSelection);
//    }

//    private void updateFolderSelection(FolderInfo root, List<String> sharedFolderPaths) {
//        if (root.isDirectory()) {
//            if (root.isSelected()) {
//                sharedFolderPaths.add(root.getPath());
//            }
//            if (root.getChildren() != null) {
//                for (FolderInfo fi: root.getChildren()) {
//                    updateFolderSelection(fi, sharedFolderPaths);
//                }
//            }
//        }
//    }
//
//    private void applySharedFolderSelection() {
//        List<FolderInfo> fiList = new ArrayList<FolderInfo>();
//        for (FolderInfo fi: sharedFolderInfo) {
//            fi.appendToList(fiList);
//        }
//        // sort 2 structs; iterate in parallel
//    }

    public void removeSharedFolder(FolderInfo selectedFolder) {
        int index = sharedFolderInfo.indexOf(selectedFolder);
        if (index != -1) {
            sharedFolderInfo.remove(index);
            sharedRoots.remove(index);
        }
    }

    public void checkCollectedFodlerInfo() {
        while (sharedFolderInfo.size() < sharedRoots.size()) {
            sharedRoots.remove(sharedRoots.size() - 1);
        }
    }

    public FolderInfo getFolderInfoByID(long id) {
        FolderInfo f = null;
        for (FolderInfo fi : sharedFolderInfo) {
            FolderInfo found = fi.getByID(id);
            if (found != null) {
                f = found;
                break;
            }
        }
        return f;
    }

    public FileListItem[] getDirListByID(long id) {
        FileListItem[] list = null;
        if (id == Consts.ROOT_FOLDER_ID) {
            list = new FileListItem[sharedFolderInfo.size()];
            int i = 0;
            for (FolderInfo fi : sharedFolderInfo) {
                list[i++] = fi.getListItem();
            }
        } else {
            FolderInfo found = getFolderInfoByID(id);
            if (found != null) {
                list = found.getDirList();
            }
        }
        return list;
    }

}
