package ru.ifmo.fileShare.util;

import java.io.FileFilter;
import java.io.File;
import java.util.List;
import java.util.ArrayList;

public class FilesEnumerator {
    public static void main(String[] args) {
        enumFoldersAsync("c:\\work\\netfileshare", new FolderEnumeratorProgress() {
            public void onFinished(int state, FolderInfo folderInfo) {
                System.out.println(folderInfo);                
            }
        });
    }

    public static void enumFoldersAsync(final String path, final FolderEnumeratorProgress progress) {
        new Thread() {
            public void run() {
                try {
                    progress.setFolderInfo(enumFolders(path, progress));
                    if (progress.isCanceled()) {
                        progress.changeState(FolderEnumeratorProgress.CANCELED);
                    } else {
                        progress.changeState(FolderEnumeratorProgress.FINISHED);
                    }
                } catch (Exception e) {
                    progress.changeState(FolderEnumeratorProgress.FAILED);
                }
            }
        }.start();
    }

    public static FolderInfo enumFolders(String path) {
        return enumFolders(path, null);
    }

    private static FolderInfo enumFolders(String path, FolderEnumeratorProgress progress) {
        if (checkCancelled(progress)) {
            return null;
        }
        File root = new File(path);
        File[] folders = root.listFiles(new FileFilter() {
            public boolean accept(File pathname) {
                return pathname.isDirectory();
            }
        });
        List<FolderInfo> children = null;
        if (folders.length > 0) {
            children = new ArrayList<FolderInfo>();
            for(File f: folders) {
                if (checkCancelled(progress)) {
                    return null;
                }
                children.add(enumFolders(f.getPath(), progress));
            }
        }
        File[] files = root.listFiles(new FileFilter() {
            public boolean accept(File pathname) {
                return pathname.isFile();
            }
        });
        for(File f: files) {
            if (children == null) {
                children = new ArrayList<FolderInfo>();
            }
            children.add(new FolderInfo(f));
        }
        if (checkCancelled(progress)) {
            return null;
        }
        return new FolderInfo(root, children);
    }

    private static boolean checkCancelled(FolderEnumeratorProgress progress) {
        return progress != null && progress.isCanceled();
    }
}
