package ru.ifmo.fileShare.util;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Properties;

public class ServerConfiguration {
    private Properties config;
    private static final String CONFIG_FILENAME = "config.properties";
    private static final String COUNT_TOKEN = ".count";

    public ServerConfiguration() {
        config = new Properties();
        load();
    }

    public void load() {
        try {
            FileInputStream fis = new FileInputStream(CONFIG_FILENAME);
            config.load(fis);
            fis.close();
        } catch (IOException e) {
            //
        }
    }

    public void save() {
        try {
            FileOutputStream fos = new FileOutputStream(CONFIG_FILENAME);
            config.store(fos, "Saved at: " + DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT).format(new Date()));
            fos.close();
        } catch (IOException e) {
            //
        }
    }

    public int getInt(String key) {
        try {
            return Integer.parseInt(config.getProperty(key));
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    public void setInt(String key, int value) {
        config.setProperty(key, String.valueOf(value));
    }

    public void setString(String key, String value) {
        config.setProperty(key, value);
    }

    public String getString(String key) {
        return config.getProperty(key);
    }

    public String[] getStringList(String key) {
        int count = getInt(key + COUNT_TOKEN);
        String[] result = new String[count];
        for (int i = 0; i < count; i++) {
            result[i] = getString(key + "." + String.valueOf(i));
        }
        return result;
    }

    public void setStringList(String key, String[] value) {
        setInt(key + COUNT_TOKEN, value.length);
        for (int i = 0; i < value.length; i++) {
            setString(key + "." + String.valueOf(i), value[i]);
        }
        // todo: remove old items
    }

    public void setStringList(String key, Collection<String> value) {
        if (value == null) {
            setInt(key + COUNT_TOKEN, 0);
        } else {
            setInt(key + COUNT_TOKEN, value.size());
            int ind = 0;
            for (String val : value) {
                setString(key + "." + String.valueOf(ind++), val);
            }
        }
        // todo: remove old items
    }
}
